package org.itqop.chatit.utils;

import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.nio.charset.StandardCharsets;
import java.util.concurrent.CompletableFuture;
import java.util.regex.Pattern;

import com.google.gson.Gson;


public class ProfanityChecker {

    private static final Gson GSON = new Gson();
    private static final HttpClient CLIENT = HttpClient.newHttpClient();

    // Ваше регулярное выражение для проверки мата
    private static final String PROFANITY_REGEX = "(?iu)\\b(?:(?:(?:у|[нз]а|(?:хитро|не)?вз?[ыьъ]|с[ьъ]|(?:и|ра)[зс]ъ?|(?:.\\B)+?[оаеи-])-?)?(?:[её](?:б(?!о[рй]|рач)|п[уа](?:ц|тс))|и[пб][ае][тцд][ьъ]).*?|(?:(?:н[иеа]|(?:ра|и)[зс]|[зд]?[ао](?:т|дн[оа])?|с(?:м[еи])?|а[пб]ч|в[ъы]?|пр[еи])-?)?ху(?:[яйиеёю]|л+и(?!ган)).*?|бл(?:[эя]|еа?)(?:[дт][ьъ]?)?|\\S*?(?:п(?:[иеё]зд|ид[аое]?р|ед(?:р(?!о)|[аое]р|ик)|охую)|бля(?:[дбц]|тс)|[ое]ху[яйиеё]|хуйн).*?|(?:о[тб]?|про|на|вы)?м(?:анд(?:[ауеыи](?:л(?:и[сзщ])?[ауеиы])?|ой|[ао]в.*?|юк(?:ов|[ауи])?|е[нт]ь|ища)|уд(?:[яаиое].+?|е?н(?:[ьюия]|ей))|[ао]л[ао]ф[ьъ](?:[яиюе]|[еёо]й))|елд[ауые].*?|ля[тд]ь|(?:[нз]а|по)х)\\b";
    private static final Pattern PROFANITY_PATTERN = Pattern.compile(PROFANITY_REGEX);

    public static CompletableFuture<Double> checkMessageAsync(String message) {
        return CompletableFuture.supplyAsync(() -> {
            try {
                String apiUrl = Config.hostApi;
                boolean useRegex = Config.useRegex;

                if (apiUrl == null || apiUrl.isEmpty()) {
                    // Если URL API не задан
                    if (useRegex) {
                        // Используем регулярное выражение для проверки мата
                        if (containsProfanity(message)) {
                            return 1.0; // Мат найден
                        } else {
                            return 0.0; // Мат не найден
                        }
                    } else {
                        // Если regex=false, возвращаем 0.0 (нет мата)
                        return 0.0;
                    }
                }

                // Создаем JSON-запрос
                String jsonRequest = GSON.toJson(new MessageRequest(message));

                HttpRequest request = HttpRequest.newBuilder()
                        .uri(URI.create(apiUrl))
                        .header("Content-Type", "application/json")
                        .POST(HttpRequest.BodyPublishers.ofString(jsonRequest, StandardCharsets.UTF_8))
                        .build();

                // Отправляем запрос и получаем ответ
                HttpResponse<String> response = CLIENT.send(request, HttpResponse.BodyHandlers.ofString());

                // Парсим ответ
                String responseBody = response.body();
                double result = Double.parseDouble(responseBody);

                return result;
            } catch (Exception e) {
                e.printStackTrace();
                if (Config.useRegex) {
                    // Если произошла ошибка и regex=true, используем регулярное выражение
                    if (containsProfanity(message)) {
                        return 1.0; // Мат найден
                    }
                }
                // В случае ошибки и regex=false считаем, что мата нет
                return 0.0;
            }
        });
    }

    private static boolean containsProfanity(String message) {
        return PROFANITY_PATTERN.matcher(message).find();
    }

    // Класс для сериализации JSON-запроса
    private static class MessageRequest {
        private final String text;

        public MessageRequest(String text) {
            this.text = text;
        }
    }
}
